function [] = open()
%OPEN Initializes the tracking parameter gui.
%   

global artoaGui artoaWorkspace;

%% Check if the gui is already opened

if isfield(artoaGui.figures, 'trackParameter')
    figure(artoaGui.figures.trackParameter);
    return
end

%% Setup callbacks
callbacks = struct();
callbacks.CloseRequestFcn = @artoa.controller.track.parameter.close;
callbacks.comboboxTrackingMethod = @artoa.controller.track.parameter.comboboxTrackingMethod;
callbacks.checkboxDopplerCorrection = @artoa.controller.track.parameter.checkboxDopplerCorrection;
callbacks.comboboxInterpolationMethod = @artoa.controller.track.parameter.comboboxInterpolationMethod;
callbacks.comboboxSoundspeedMethod = @artoa.controller.track.parameter.comboboxSoundspeedMethod;
callbacks.checkboxPlotResiduals = @artoa.controller.track.parameter.checkboxPlotResiduals;
callbacks.inputOutputInterpolationInterval = @artoa.controller.track.parameter.inputOutputInterpolationInterval;
callbacks.inputGapSize = @artoa.controller.track.parameter.inputGapSize;
callbacks.tableSoundSourceCombinationsSelection = @artoa.controller.track.parameter.tableSoundsourceCombinationsSelection;
callbacks.tableSoundSourceCombinationsEdit = @artoa.controller.track.parameter.tableSoundsourceCombinationsEdit;
callbacks.tableSoundSourceOffsetsEdit = @artoa.controller.track.parameter.tableSoundsourceOffsetsEdit;
callbacks.buttonTrack = @artoa.controller.track.run;
callbacks.buttonQuickTrack = '';
callbacks.buttonAddCombination = @artoa.controller.track.parameter.addSoundsourceCombination;
callbacks.buttonAddCombinationAbove = @artoa.controller.track.parameter.addSoundsourceCombinationAbove;
callbacks.buttonDuplicateCombination = @artoa.controller.track.parameter.duplicateSoundsourceCombination;
callbacks.buttonRemoveCombination = @artoa.controller.track.parameter.removeSoundsourceCombination;
callbacks.buttonResetAllCombinations = @artoa.controller.track.parameter.resetSoundsourceCombinations;
callbacks.inputFloatOffsetBegin = @artoa.controller.track.parameter.inputFloatOffsetBegin;
callbacks.inputFloatOffsetEnd = @artoa.controller.track.parameter.inputFloatOffsetEnd;

%% Open the gui
artoa.gui.track.parameter( ...
    callbacks, ...
    artoaWorkspace.defaults.trackingMethods, ...
    artoaWorkspace.defaults.interpolationMethods, ...
    artoaWorkspace.defaults.soundspeedMethods ...
);

%% Get involved
soundsourceNames = fieldnames(artoaWorkspace.filteredSoundsources);
appliedSoundsources = {};
involvedSoundsources = struct();
for i = 1:length(soundsourceNames)
    if artoa.soundsources.hasAppliedToa(soundsourceNames{i}, artoaWorkspace.toaData.soundSource)
        extracted = artoa.data.extractSoundsourcesFromStruct( ...
            soundsourceNames{i}, ...
            artoaWorkspace.filteredSoundsources ...
        );
        involvedSoundsources.(soundsourceNames{i}) = extracted;
        clear extracted;
    end
end

%% Setup sound source offsets table
% initialize sound source offsets
artoaGui.trackParameter.tableSoundSourceOffsets.RowName = fieldnames( ...
    involvedSoundsources ...
);


%% Create workspace variables
if ~isfield(artoaWorkspace, 'trackParameter')
    artoaWorkspace.trackParameter = struct();
    
    % setup sound source combinations
    artoaWorkspace.trackParameter.soundsourceCombinations = ...
        artoa.soundsources.createCombinationsTable(...
            artoa.controller.track.parameter.getDefaultCombinationCell() ...
    );
    artoaGui.trackParameter.tableSoundSourceCombinations.Data = artoa.controller.track.parameter.getDefaultCombinationCell();
    
    % initialize sound source offsets
    artoaGui.trackParameter.tableSoundSourceOffsets.Data(1:length(artoaGui.trackParameter.tableSoundSourceOffsets.RowName), 1) = {0};
    artoaWorkspace.trackParameter.soundsourceOffsets = artoa.soundsources.createOffsetsStruct( ...
        artoaGui.trackParameter.tableSoundSourceOffsets.RowName, ...
        artoaGui.trackParameter.tableSoundSourceOffsets.Data ...
    );
    
    % initialize settings
    availableCallbacks = { ...
        'comboboxTrackingMethod', ...
        'checkboxDopplerCorrection', ...
        'comboboxInterpolationMethod', ...
        'comboboxSoundspeedMethod', ...
        'checkboxPlotResiduals', ...
        'inputOutputInterpolationInterval', ...
        'inputGapSize', ...
        'tableSoundSourceOffsetsEdit', ...
        'tableSoundSourceCombinationsEdit', ...
        'inputFloatOffsetBegin', ...
        'inputFloatOffsetEnd' ...
    };

    for i = 1:length(availableCallbacks)
        if isfield(callbacks, availableCallbacks{i})
            % execute the callback to save the state
            callbacks.(availableCallbacks{i})();
        end
    end
end

%% Update the values if required
artoa.controller.track.parameter.updateGui();

end

